"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var R = require("ramda");
var util_1 = require("../../../util");
var textfield_1 = require("../textfield");
var dialog_1 = require("../../dialog");
var addressfield_interface_1 = require("./addressfield.interface");
var di_1 = require("../../../di");
var google_1 = require("../../google");
var AddressField = /** @class */ (function (_super) {
    __extends(AddressField, _super);
    /**
     * @stable [29.07.2018]
     * @param {IAddressFieldProps} props
     */
    function AddressField(props) {
        var _this = _super.call(this, props) || this;
        if (!props.notUsePlaceAction) {
            _this.addMapAction();
        }
        _this.onMenuSelect = _this.onMenuSelect.bind(_this);
        _this.onMarkerChangePlace = _this.onMarkerChangePlace.bind(_this);
        _this.onPlaceChanged = _this.onPlaceChanged.bind(_this);
        _this.onDialogClose = _this.onDialogClose.bind(_this);
        _this.onDialogAccept = _this.onDialogAccept.bind(_this);
        _this.initGoogleMapsObjects = _this.initGoogleMapsObjects.bind(_this);
        // Don't need to save this values to React State, because UI doesn't use these bindings !!
        // => Need to initialize the internal "state" manually
        _this.lat = props.lat;
        _this.lng = props.lng;
        return _this;
    }
    /**
     * @stable [30.07.2018]
     */
    AddressField.prototype.componentWillUnmount = function () {
        _super.prototype.componentWillUnmount.call(this);
        this.cancelGeoCoderTaskIfPending();
        this.autocomplete = null;
        this.geoCoderTask = null;
        this.clearInternalVariables();
    };
    /**
     * @stable [30.07.2018]
     * @param {AnyT} event
     */
    AddressField.prototype.onChange = function (event) {
        this.resetAllStateOnChangeOrClearValue();
        _super.prototype.onChange.call(this, event);
    };
    /**
     * @stable [30.07.2018]
     */
    AddressField.prototype.clearValue = function () {
        this.resetAllStateOnChangeOrClearValue();
        _super.prototype.clearValue.call(this);
    };
    /**
     * @stable [31.07.2018]
     * @returns {JSX.Element}
     */
    AddressField.prototype.getAttachmentElement = function () {
        var state = this.state;
        var messages = this.settings.messages;
        var currentPlace = R.isNil(state.place) ? this.value : state.place;
        return (React.createElement(dialog_1.Dialog, { ref: 'dialog', title: messages.addressSelectionMessage, acceptDisabled: !this.isPlaceChanged, closeMessage: messages.closeMessage, acceptMessage: messages.acceptMessage, onClose: this.onDialogClose, onAccept: this.onDialogAccept },
            util_1.orNull(currentPlace, function () { return React.createElement("div", { className: 'rac-dialog-address-field-place' }, currentPlace); }),
            React.createElement(google_1.GoogleMaps, { ref: 'googleMaps', onInit: this.initGoogleMapsObjects, onSelect: this.onMenuSelect, onChangePlace: this.onMarkerChangePlace, options: [{
                        label: this.settings.messages.putMarkerHereMessage,
                        value: addressfield_interface_1.AddressMapMarkerActionEnum.PUT_MARKER_HERE,
                    }] })));
    };
    /**
     * @stable [01.08.2018]
     * @returns {string | number}
     */
    AddressField.prototype.buildValue = function () {
        if (this.props.useZipCode) {
            return this.placeEntity.zipCode;
        }
        return util_1.toAddress(this.placeEntity);
    };
    /**
     * @stable [01.08.2018]
     * @param {string} value
     * @returns {string}
     */
    AddressField.prototype.validateValue = function (value) {
        if (this.props.notUseCustomValidator) {
            return null;
        }
        if (R.isNil(this.lat) && R.isNil(this.lng) && !R.isNil(value) && !R.isEmpty(value)) {
            return this.settings.messages.invalidAddressMessage;
        }
        return null;
    };
    /**
     * @stable [31.07.2018]
     */
    AddressField.prototype.onPlaceChanged = function () {
        var place = this.autocomplete.getPlace();
        if (!place || !place.geometry) {
            this.clearInternalVariables(); // Clear the internal variables if error
            return;
        }
        this.lat = place.geometry.location.lat();
        this.lng = place.geometry.location.lng();
        this.placeId = place.place_id;
        this.placeEntity = util_1.toPlace(place);
        this.onChangeManually(this.buildValue()); // Emit event to update a component value
        this.propsChangePlace(); // Emit event to update a side values
    };
    /**
     * @stable [31.07.2018]
     */
    AddressField.prototype.onDialogAccept = function () {
        this.onChangeManually(this.state.place); // Emit event to update a component value
        this.propsChangePlace(); // Emit event to update a side values
        this.clearInternalVariables(); // Clear the internal variables
    };
    /**
     * @stable [31.07.2018]
     */
    AddressField.prototype.onDialogClose = function () {
        this.clearInternalVariables(); // Clear the internal variables
    };
    /**
     * @stable [31.07.2018]
     */
    AddressField.prototype.propsChangePlace = function () {
        var props = this.props;
        if (props.onChangePlace) {
            props.onChangePlace({
                lat: this.lat,
                lng: this.lng,
                placeId: this.placeId,
                placeEntity: this.placeEntity,
            });
        }
    };
    /**
     * @stable [31.07.2018]
     */
    AddressField.prototype.initGoogleMapsObjects = function () {
        this.googleMaps.addMarker({ draggable: true, position: null }, AddressField.ADDRESS_MARKER);
        this.autocomplete = new google.maps.places.Autocomplete(this.input, {
            componentRestrictions: this.settings.companyCountry ? { country: this.settings.companyCountry } : null,
        });
        this.autocomplete.addListener('place_changed', this.onPlaceChanged);
    };
    /**
     * @stable [31.07.2018]
     * @param {IGoogleMapsSelectEntity} payload
     */
    AddressField.prototype.onMenuSelect = function (payload) {
        this.lat = payload.lat;
        this.lng = payload.lng;
        switch (payload.item.value) {
            case addressfield_interface_1.AddressMapMarkerActionEnum.PUT_MARKER_HERE:
                this.googleMaps.setMarkerState({
                    marker: AddressField.ADDRESS_MARKER,
                    visibility: true,
                    refreshMap: false,
                    lat: this.lat,
                    lng: this.lng,
                });
                this.updateGeocodeInfo();
                break;
        }
    };
    /**
     * @stable [31.07.2018]
     * @param {IGoogleMapsMarkerChangePlaceEntity} place
     */
    AddressField.prototype.onMarkerChangePlace = function (place) {
        this.lat = place.lat;
        this.lng = place.lng;
        this.updateGeocodeInfo();
    };
    Object.defineProperty(AddressField.prototype, "isPlaceChanged", {
        /**
         * @stable [30.07.2018]
         * @returns {boolean}
         */
        get: function () {
            return !R.isNil(this.state.place);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [31.07.2018]
     */
    AddressField.prototype.updateGeocodeInfo = function () {
        var _this = this;
        this.cancelGeoCoderTaskIfPending();
        this.geoCoderTask = this.geoCoder.geocode({ location: { lat: this.lat, lng: this.lng } })
            .then(function (geocoderResults) { return _this.onGeocodeInfoLoad(geocoderResults); });
    };
    /**
     * @stable [31.07.2018]
     */
    AddressField.prototype.cancelGeoCoderTaskIfPending = function () {
        this.cancelTaskIfPending(this.geoCoderTask);
    };
    /**
     * @stable [31.07.2018]
     */
    AddressField.prototype.cancelTaskIfPending = function (promise) {
        if (promise && promise.isPending()) {
            promise.cancel();
        }
    };
    /**
     * @stable [31.07.2018]
     * @param {google.maps.GeocoderResult[]} geocoderResults
     * @returns {google.maps.GeocoderResult[]}
     */
    AddressField.prototype.onGeocodeInfoLoad = function (geocoderResults) {
        if (!Array.isArray(geocoderResults) || geocoderResults.length === 0) {
            return null;
        }
        var place = geocoderResults[0];
        this.placeId = place.place_id;
        this.placeEntity = util_1.toPlace(place);
        this.setState({ place: this.buildValue() });
        return geocoderResults;
    };
    /**
     * @stable [31.07.2018]
     */
    AddressField.prototype.resetAllStateOnChangeOrClearValue = function () {
        this.clearInternalVariables();
        this.propsChangePlace();
    };
    /**
     * @stable [30.07.2018]
     */
    AddressField.prototype.clearInternalVariables = function () {
        delete this.lat;
        delete this.lng;
        delete this.placeId;
        delete this.placeEntity;
        this.setState({ place: null });
    };
    /**
     * @stable [29.07.2018]
     */
    AddressField.prototype.addMapAction = function () {
        var this0 = this;
        this.defaultActions = R.insert(0, {
            type: 'location_on',
            onClick: function () {
                this0.openDialog();
            },
        }, this.defaultActions);
    };
    /**
     * @stable [30.07.2018]
     */
    AddressField.prototype.openDialog = function () {
        this.dialog.activate();
        var props = this.props;
        var isMarkerVisible = !(R.isNil(props.lat) && R.isNil(props.lng));
        if (isMarkerVisible) {
            this.googleMaps.setMarkerState({
                marker: AddressField.ADDRESS_MARKER,
                visibility: true,
                refreshMap: true,
                lat: props.lat,
                lng: props.lng,
                zoom: this.settings.googleMaps.prettyZoom,
            });
        }
        else {
            this.googleMaps.setMarkerState({
                marker: AddressField.ADDRESS_MARKER,
                visibility: false,
                refreshMap: true,
            });
        }
    };
    Object.defineProperty(AddressField.prototype, "dialog", {
        /**
         * @stable [29.07.2018]
         * @returns {IUniversalDialog}
         */
        get: function () {
            return this.refs.dialog;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AddressField.prototype, "googleMaps", {
        /**
         * @stable [31.07.2018]
         * @returns {IGoogleMaps}
         */
        get: function () {
            return this.refs.googleMaps;
        },
        enumerable: true,
        configurable: true
    });
    AddressField.defaultProps = {
        placeholder: ' ',
    };
    AddressField.ADDRESS_MARKER = util_1.uuid();
    __decorate([
        di_1.lazyInject(di_1.DI_TYPES.GeoCoder),
        __metadata("design:type", Object)
    ], AddressField.prototype, "geoCoder", void 0);
    return AddressField;
}(textfield_1.BaseTextField));
exports.AddressField = AddressField;
//# sourceMappingURL=addressfield.component.js.map