"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var definitions_interface_1 = require("../../../definitions.interface");
var util_1 = require("../../../util");
/**
 * @stable [18.08.2018]
 * @param {MultiFieldEntityT} entity
 * @returns {TItem[]}
 */
exports.toActualMultiItemEntities = function (entity) {
    if (R.isNil(entity)) {
        return definitions_interface_1.UNDEF;
    }
    var multiEntity = entity;
    if (!exports.isNotMultiEntity(entity)) {
        var editedEntities = exports.toActualMultiItemEditedEntities(entity);
        var originalSourceItems_1 = multiEntity.source || [];
        var actualSourceItems = R.filter(function (entity0) { return (
        // Exclude the removed entities from original snapshot
        !multiEntity.remove.find(function (removeId) { return removeId.id === entity0.id; })
            // Exclude the edited entities from an original snapshot,
            // because we need to replace them with actual edited entities
            && !multiEntity.edit.find(function (editedId) { return editedId.id === entity0.id; })); }, originalSourceItems_1);
        // The final snapshot
        var entities = actualSourceItems.concat(multiEntity.add).concat(editedEntities);
        // Finally, need to sorting by original entity position in an original snapshot
        return R.sort(function (item1, item2) {
            var firstIndex = originalSourceItems_1.findIndex(function (i) { return i.id === item1.id; });
            var secondIndex = originalSourceItems_1.findIndex(function (i) { return i.id === item2.id; });
            return firstIndex > secondIndex ? 1 : (firstIndex === secondIndex ? 0 : -1);
        }, entities);
    }
    return entity;
};
/**
 * @stable [22.11.2018]
 * @param {MultiFieldEntityT} entity
 * @returns {TItem[]}
 */
exports.toActualMultiItemDeletedEntities = function (entity) {
    if (R.isNil(entity)) {
        return definitions_interface_1.UNDEF;
    }
    var multiEntity = entity;
    if (!exports.isNotMultiEntity(entity)) {
        return multiEntity.remove;
    }
    return [];
};
/**
 * @stable [18.08.2018]
 * @param {MultiFieldEntityT} entity
 * @returns {TItem[]}
 */
exports.toActualMultiItemEditedEntities = function (entity) {
    if (R.isNil(entity)) {
        return definitions_interface_1.UNDEF;
    }
    var multiEntity = entity;
    if (!exports.isNotMultiEntity(entity)) {
        var editedItems_1 = {};
        multiEntity.edit.forEach(function (editedItem) {
            var editedItemId = editedItem.id;
            var editedItem0 = editedItems_1[editedItemId] || __assign({}, multiEntity.source.find(function (originalEntity) { return originalEntity.id === editedItemId; }));
            editedItem0[editedItem.name] = editedItem.value;
            editedItems_1[editedItemId] = editedItem0;
        });
        return Object.keys(editedItems_1).map(function (editedItemId) { return editedItems_1[editedItemId]; });
    }
    return [];
};
/**
 * @stable [26.12.2018]
 * @param {MultiFieldEntityT} entity
 * @returns {TItem[]}
 */
exports.toActualMultiItemAddedEntities = function (entity) {
    if (R.isNil(entity)) {
        return definitions_interface_1.UNDEF;
    }
    var multiEntity = entity;
    return exports.isNotMultiEntity(entity) ? [] : multiEntity.add;
};
/**
 * @stable [27.06.2018]
 * @param {MultiFieldSingleValueT} multiFieldEntity
 * @returns {string}
 */
exports.toLastAddedMultiItemEntity = function (multiFieldEntity) {
    if (R.isNil(multiFieldEntity)) {
        return null;
    }
    if (util_1.isPrimitive(multiFieldEntity)) {
        return String(multiFieldEntity);
    }
    var valueAsMultiEntity = multiFieldEntity;
    var add = valueAsMultiEntity.add;
    if (R.isNil(add)) {
        return null;
    }
    return util_1.orNull(add.length > 0, function () { return String(add[add.length - 1].id); });
};
/**
 * @stable [26.06.2018]
 * @param {MultiFieldEntityT} multiFieldEntity
 * @returns {EntityIdT[]}
 */
exports.fromMultiFieldEntityToEntitiesIds = function (multiFieldEntity) {
    return fromMultiFieldEntityToEntities(multiFieldEntity, function (entity) { return entity.id; });
};
/**
 * @stable [26.06.2018]
 * @param {MultiFieldEntityT} multiFieldEntity
 * @param {(entity: TItem, index: number) => TResult} mapper
 * @returns {TResult[]}
 */
function fromMultiFieldEntityToEntities(multiFieldEntity, mapper) {
    var result = exports.toActualMultiItemEntities(multiFieldEntity);
    return util_1.orUndef(!R.isNil(result), function () { return result.map(mapper); });
}
exports.fromMultiFieldEntityToEntities = fromMultiFieldEntityToEntities;
/**
 * @stable [04.07.2018]
 * @param {MultiFieldEntityT} multiFieldEntity
 * @returns {EntityIdT[]}
 */
exports.fromMultiFieldEntityToEditedEntitiesIds = function (multiFieldEntity) {
    return fromMultiFieldEntityToEditedEntities(multiFieldEntity, function (entity) { return entity.id; });
};
/**
 * @stable [18.08.2018]
 * @param {MultiFieldEntityT} multiFieldEntity
 * @param {(entity: TItem, index: number) => TResult} mapper
 * @returns {TResult[]}
 */
function fromMultiFieldEntityToEditedEntities(multiFieldEntity, mapper) {
    var result = exports.toActualMultiItemEditedEntities(multiFieldEntity);
    return util_1.orUndef(!R.isNil(result), function () { return result.map(mapper); });
}
exports.fromMultiFieldEntityToEditedEntities = fromMultiFieldEntityToEditedEntities;
/**
 * @stable [22.11.2018]
 * @param {MultiFieldEntityT} multiFieldEntity
 * @param {(entity: TItem, index: number) => TResult} mapper
 * @returns {TResult[]}
 */
function fromMultiFieldEntityToDeletedEntities(multiFieldEntity, mapper) {
    var result = exports.toActualMultiItemDeletedEntities(multiFieldEntity);
    return util_1.orUndef(!R.isNil(result), function () { return result.map(mapper); });
}
exports.fromMultiFieldEntityToDeletedEntities = fromMultiFieldEntityToDeletedEntities;
/**
 * @stable [24.06.2018]
 * @param {MultiFieldEntityT | EntityIdT} value
 * @returns {number}
 */
exports.toActualMultiItemEntitiesLength = function (value) {
    return util_1.orDefault(util_1.isDef(value), function () { return ((exports.isNotMultiEntity(value)
        ? exports.normalizeEntities(value)
        : exports.toActualMultiItemEntities(value)).length); }, 0);
};
/**
 * @stable [03.07.2018]
 * @param {NotMultiFieldEntityT} value
 * @returns {IEntity[]}
 */
exports.normalizeEntities = function (value) {
    return util_1.isPrimitive(value) ? [{ id: value }] : value;
};
/**
 * @stable [24.06.2018]
 * @param {MultiFieldEntityT | EntityIdT} value
 * @returns {boolean}
 */
exports.isNotMultiEntity = function (value) {
    return Array.isArray(value) || util_1.isPrimitive(value);
};
/**
 * @stable [23.06.2018]
 * @param {MultiFieldEntityT} value
 * @param {(value: IMultiEntity) => IMultiItemEntity[]} converter
 * @param {IEntity[]} defaultValue
 * @returns {IMultiItemEntity[] | IEntity[]}
 */
exports.extractMultiItemEntities = function (value, converter, defaultValue) {
    return exports.isNotMultiEntity(value)
        ? util_1.orDefault(util_1.isDef(defaultValue), defaultValue, function () { return exports.normalizeEntities(value); })
        : (R.isNil(value) ? [] : converter(value));
};
/**
 * @stable [23.06.2018]
 * @param {MultiFieldEntityT} value
 * @param {IMultiItemEntity[]} defaultValue
 * @returns {IMultiItemEntity[]}
 */
exports.extractMultiEditItemEntities = function (value, defaultValue) {
    if (defaultValue === void 0) { defaultValue = []; }
    return exports.extractMultiItemEntities(value, function (currentValue) { return currentValue.edit; }, defaultValue);
};
/**
 * @stable [23.06.2018]
 * @param {MultiFieldEntityT} value
 * @param {IMultiItemEntity[]} defaultValue
 * @returns {IMultiItemEntity[]}
 */
exports.extractMultiRemoveItemEntities = function (value, defaultValue) {
    if (defaultValue === void 0) { defaultValue = []; }
    return exports.extractMultiItemEntities(value, function (currentValue) { return currentValue.remove; }, defaultValue);
};
/**
 * @stable [02.07.2018]
 * @param {MultiFieldEntityT} value
 * @param {IMultiItemEntity[]} defaultValue
 * @returns {IEntity[]}
 */
exports.extractMultiAddItemEntities = function (value, defaultValue) {
    if (defaultValue === void 0) { defaultValue = []; }
    return exports.extractMultiItemEntities(value, function (currentValue) { return currentValue.add; }, defaultValue);
};
/**
 * @stable [23.06.2018]
 * @param {MultiFieldEntityT} value
 * @param {IEntity[]} defaultValue
 * @returns {IEntity[]}
 */
exports.extractMultiSourceItemEntities = function (value, defaultValue) {
    return exports.extractMultiItemEntities(value, function (currentValue) { return currentValue.source; }, defaultValue);
};
/**
 * @stable [02.07.2018]
 * @param {IEntity | IMultiItemEntity} entity
 * @returns {IEntity}
 */
exports.fromMultiItemEntityToEntity = function (entity) {
    var _a;
    var entityAsMultiItemEntity = entity;
    if (util_1.isDef(entityAsMultiItemEntity.rawData)) {
        return __assign({}, entityAsMultiItemEntity.rawData, (_a = {}, _a[entityAsMultiItemEntity.name] = entityAsMultiItemEntity.value, _a));
    }
    return entity;
};
/**
 * @stable [02.07.2018]
 * @param {string} name
 * @param {AnyT} value
 * @param {IEntity} rawData
 * @param {boolean} newEntity
 * @returns {IMultiItemEntity}
 */
exports.buildMultiEntity = function (name, value, rawData, newEntity) {
    return util_1.defValuesFilter({ id: rawData.id, value: value, name: name, rawData: rawData, newEntity: newEntity });
};
/**
 * @stable [23.06.2018]
 * @param {string} fieldName
 * @param {MultiFieldEntityT} multiFieldValue
 * @param {(itm: IMultiItemEntity) => boolean} predicate
 * @param {(itm: IMultiItemEntity) => AnyT} nextFieldValueFn
 * @returns {IMultiItemEntity}
 */
exports.buildMultiEditItemEntityPayload = function (fieldName, multiFieldValue, predicate, nextFieldValueFn) {
    var sourceMultiItemEntities = exports.extractMultiSourceItemEntities(multiFieldValue);
    var editedMultiItemEntities = exports.extractMultiEditItemEntities(multiFieldValue);
    var editedMultiItemEntity = editedMultiItemEntities.find(predicate);
    var sourceMultiItemEntity = sourceMultiItemEntities.find(predicate);
    return exports.buildMultiEntity(fieldName, nextFieldValueFn(editedMultiItemEntity, sourceMultiItemEntity), sourceMultiItemEntity);
};
/**
 * @stable [02.07.2018]
 * @param {string} fieldName
 * @param {MultiFieldEntityT} multiFieldValue
 * @param {(itm: IMultiItemEntity) => boolean} predicate
 * @param {(itm: TEntity) => AnyT} nextFieldValueFn
 * @param {(newEntity: boolean, entity: TEntity) => TEntity} entityFactory
 * @returns {IMultiItemEntity}
 */
exports.buildMultiAddItemEntityPayload = function (fieldName, multiFieldValue, predicate, nextFieldValueFn, entityFactory) {
    var addedMultiItemEntities = exports.extractMultiAddItemEntities(multiFieldValue);
    var addedEntity = addedMultiItemEntities.find(predicate);
    var newEntity = R.isNil(addedEntity);
    return exports.buildMultiEntity(fieldName, nextFieldValueFn(addedEntity), util_1.isFn(entityFactory) ? entityFactory(newEntity, addedEntity) : addedEntity, newEntity);
};
/**
 * @stable [04.07.2018]
 * @param {IMultiItemEntity[]} multiItemEntities
 * @returns {IKeyValue}
 */
exports.fromMultiItemEntitiesToFieldsChanges = function (multiItemEntities) {
    return R.reduceBy(function (acc, entity) {
        var _a;
        return (__assign({}, acc, (_a = {}, _a[entity.name] = entity.value, _a)));
    }, {}, function (entity) { return String(entity.id); }, multiItemEntities);
};
/**
 * @stable [11.08.2018]
 * @param {IMultiItemEntity} first
 * @param {IMultiItemEntity} second
 * @returns {boolean}
 */
exports.isSameEntityByIdAndChangedFieldName = function (first, second) { return (first.id === second.id
    && first.name === second.name); };
/**
 * @stable [11.08.2018]
 * @param {IMultiItemEntity} item
 * @param {IEntity[]} addValue
 * @param {IMultiItemEntity[]} removeValue
 * @param {IMultiItemEntity[]} editValue
 * @param {IEntity[]} originalValue
 * @returns {IMultiFieldChangesEntity}
 */
exports.toMultiFieldChangesEntityOnEdit = function (item, addValue, removeValue, editValue, originalValue) {
    var removeArray = removeValue;
    var originalEditedItem = originalValue.find(function (originalItem0) { return originalItem0.id === item.id; });
    var editedNewItem = addValue.find(function (editedNewItem0) { return editedNewItem0.id === item.id; });
    var isEditedNewItem = !R.isNil(editedNewItem);
    var isOriginalEditedItem = !R.isNil(originalEditedItem);
    var editArray = util_1.orDefault(isEditedNewItem, editValue, // If a user is editing a new record then returning an input value
    function () { return ( // Otherwise, we should replace an old item with an input item
    editValue
        .filter(function (editedItem) { return !exports.isSameEntityByIdAndChangedFieldName(item, editedItem); })
        .concat(item)
        .filter(function (editedItem) {
        // Totally destroy the dirty changes, if an edited entity attribute is equal an original entity attribute
        return !isOriginalEditedItem
            || !(exports.isSameEntityByIdAndChangedFieldName(item, editedItem)
                && originalEditedItem[item.name] === item.value);
    })); });
    var addArray = util_1.orDefault(isEditedNewItem, function () { return (item.value === definitions_interface_1.UNDEF // Need to destroy added entity
        ? R.filter(function (itm) { return itm.id !== item.id; }, addValue)
        : R.map(function (newItem) { return newItem.id === item.id ? exports.fromMultiItemEntityToEntity(item) : newItem; }, addValue)); }, addValue);
    return { addArray: addArray, removeArray: removeArray, editArray: editArray };
};
/**
 * @stable [18.08.2018]
 * @param {IMultiItemEntity} item
 * @param {IEntity[]} addValue
 * @param {IMultiItemEntity[]} removeValue
 * @param {IMultiItemEntity[]} editValue
 * @param {IEntity[]} originalValue
 * @returns {IMultiFieldChangesEntity}
 */
exports.toMultiFieldChangesEntityOnDelete = function (item, addValue, removeValue, editValue, originalValue) {
    var deletedItemId = item.id;
    var addValueLenBeforeFiltering = addValue.length;
    var editArray = editValue.filter((function (addItem) { return addItem.id !== deletedItemId; }));
    var addArray = addValue.filter((function (addItem) { return addItem.id !== deletedItemId; }));
    var removeArray = removeValue;
    if (addArray.length === addValueLenBeforeFiltering) {
        // The local added items do not contain deleted item
        var deletedEntity = originalValue.find(function (entity) { return entity.id === deletedItemId; });
        removeArray = (R.isNil(deletedEntity) ? [] : [deletedEntity]).concat(removeValue);
    }
    else {
        // We just now have removed deleted item from addValue array
    }
    return { addArray: addArray, removeArray: removeArray, editArray: editArray };
};
//# sourceMappingURL=multifield.support.js.map