"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var operation_1 = require("../../operation");
var util_1 = require("../../util");
var tabpanel_support_1 = require("../tabpanel/tabpanel.support");
/**
 * TODO api support buildApiEntity
 * @param {IEntity} changes
 * @param {IEntity} entity
 * @param {IEntity} originalEntity
 * @param {string} operationId
 * @returns {IApiEntity}
 */
exports.buildApiEntity = function (changes, entity, originalEntity, operationId) {
    var entityId = entity && entity.id;
    var merger = __assign({}, entity, changes);
    var apiEntity = (R.isNil(entityId)
        // You should use formMapper at least (simple form)
        ? { isNew: true, changes: __assign({}, changes), merger: merger, }
        // You should use formMapper and entityMapper at least (editable entity)
        : util_1.defValuesFilter({
            isNew: false,
            id: entityId,
            changes: __assign({}, changes),
            merger: merger,
            entity: R.isNil(entity) ? entity : __assign({}, entity),
            originalEntity: R.isNil(originalEntity) ? originalEntity : __assign({}, originalEntity),
        }));
    return __assign({ operation: operation_1.Operation.create(operationId) }, apiEntity);
};
/**
 * @stable [29.05.2018]
 * @param {IFormProps} formProps
 * @param {IFieldConfiguration} fieldProps
 * @returns {boolean}
 */
exports.isFormFieldReadOnly = function (formProps, fieldProps) {
    return (R.isNil(fieldProps.readOnly) ? formProps.readOnly : fieldProps.readOnly) === true;
};
/**
 * @stable [29.05.2018]
 * @param {IFormProps} formProps
 * @param {IFieldConfiguration} fieldProps
 * @returns {boolean}
 */
exports.isFormFieldDisabled = function (formProps, fieldProps) {
    return R.isNil(fieldProps.disabled)
        ? exports.isFormDisabled(formProps)
        : fieldProps.disabled === true;
};
/**
 * @stable [16.11.2018]
 * @param {IFormProps} formProps
 * @param {IFieldConfiguration} fieldProps
 * @returns {boolean}
 */
exports.isFormFieldChangeable = function (formProps, fieldProps) {
    return R.isNil(fieldProps.changeable)
        ? exports.isFormChangeable(formProps)
        : fieldProps.changeable !== false;
};
/**
 * @stable [29.05.2018]
 * @param {IFormProps} formProps
 * @returns {boolean}
 */
exports.isFormDisabled = function (formProps) {
    return formProps.disabled === true || formProps.form.progress === true;
};
/**
 * @stable [16.11.2018]
 * @param {IFormProps} formProps
 * @returns {boolean}
 */
exports.isFormChangeable = function (formProps) { return formProps.changeable !== false; };
/**
 * @stable [03.08.2018]
 * @param {IFormProps} formProps
 * @returns {boolean}
 */
exports.isFormValid = function (formProps) {
    return R.isNil(formProps.form.valid) || formProps.form.valid === true;
};
/**
 * @stable [29.05.2018]
 * @param {IFormProps} formProps
 * @returns {boolean}
 */
exports.isFormEditable = function (formProps) {
    return R.isNil(formProps.editable) || formProps.editable === true;
};
/**
 * @stable [29.05.2018]
 * @param {IFormProps} formProps
 * @returns {boolean}
 */
exports.isFormDirty = function (formProps) {
    return formProps.alwaysDirty || formProps.form.dirty === true;
};
/**
 * @stable [03.08.2018]
 * @param {IFormProps} formProps
 * @returns {boolean}
 */
exports.isFormSubmittable = function (formProps) {
    return exports.isFormValid(formProps)
        && exports.isFormEditable(formProps)
        && exports.isFormDirty(formProps)
        && !exports.isFormDisabled(formProps);
};
/**
 * @stable [15.02.2019]
 * @param {IFormProps} formProps
 * @returns {boolean}
 */
exports.isFormResettable = function (formProps) { return formProps.alwaysResettable || exports.isFormDirty(formProps); };
/**
 * @stable [29.05.2018]
 * @param {IFormProps} formProps
 * @returns {boolean}
 */
exports.isFormNewEntity = function (formProps) { return R.isNil(formProps.entity) || R.isNil(formProps.entity.id); };
/**
 * @stable [30.08.2018]
 * @param {IFormProps} formProps
 * @param {ITabConfiguration} tab
 * @returns {boolean}
 */
exports.isFormTabActive = function (formProps, tab) {
    return tabpanel_support_1.isTabActive(formProps.form, tab);
};
/**
 * @stable [30.08.2018]
 * @param {IFormProps} formProps
 * @param {ITabPanelConfiguration} tabPanelConfiguration
 * @returns {number}
 */
exports.getFormTabActiveValue = function (formProps, tabPanelConfiguration) {
    return tabpanel_support_1.getTabActiveValue(formProps.form, tabPanelConfiguration);
};
//# sourceMappingURL=form.support.js.map