"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
/**
 * @stable [31.08.2018]
 * @param {IEffectsAction} action
 * @param {string} alias
 * @param {ITabPanelConfiguration} tabPanelConfiguration
 * @returns {boolean}
 */
exports.isTabActivated = function (action, alias, tabPanelConfiguration) {
    var payloadWrapper = action.data;
    return payloadWrapper.payload === exports.findTabValueByAlias(alias, tabPanelConfiguration);
};
/**
 * @stable [31.08.2018]
 * @param {string} alias
 * @param {ITabPanelConfiguration} tabPanelConfiguration
 * @returns {ITabConfiguration}
 */
exports.findTabByAlias = function (alias, tabPanelConfiguration) {
    return tabPanelConfiguration.items.find(function (tab) { return tab.alias === alias; });
};
/**
 * @stable [31.08.2018]
 * @param {string} alias
 * @param {ITabPanelConfiguration} tabPanelConfiguration
 * @returns {ITabConfiguration}
 */
exports.findTabValueByAlias = function (alias, tabPanelConfiguration) {
    return exports.findTabByAlias(alias, tabPanelConfiguration).value;
};
/**
 * @stable [31.08.2018]
 * @param {IActiveValueWrapper} activeValueWrapper
 * @param {ITabPanelConfiguration} tabPanelConfiguration
 * @param {string} alias
 * @returns {boolean}
 */
exports.isTabActiveByAlias = function (activeValueWrapper, tabPanelConfiguration, alias) {
    return exports.isTabActive(activeValueWrapper, exports.findTabByAlias(alias, tabPanelConfiguration));
};
/**
 * @stable [31.08.2018]
 * @param {ITabPanelWrapperEntity} tabPanelWrapperEntity
 * @param {ITabPanelConfiguration} tabPanelConfiguration
 * @param {string} alias
 * @returns {boolean}
 */
exports.isTabPanelWrapperEntityActiveByAlias = function (tabPanelWrapperEntity, tabPanelConfiguration, alias) {
    return exports.isTabActiveByAlias(tabPanelWrapperEntity.tabPanel, tabPanelConfiguration, alias);
};
/**
 * @stable [30.08.2018]
 * @param {IActiveValueWrapper} activeValueWrapper
 * @param {ITabConfiguration} tab
 * @returns {boolean}
 */
exports.isTabActive = function (activeValueWrapper, tab) {
    return R.isNil(activeValueWrapper.activeValue) ? !!tab.active : activeValueWrapper.activeValue === tab.value;
};
/**
 * @stable [30.08.2018]
 * @param {IActiveValueWrapper} props
 * @param {ITabPanelConfiguration} tabPanelConfiguration
 * @returns {number}
 */
exports.getTabActiveValue = function (props, tabPanelConfiguration) {
    var currentActiveValue = props.activeValue;
    if (!R.isNil(currentActiveValue)) {
        return currentActiveValue;
    }
    var tabs = tabPanelConfiguration.items;
    var activeTab = tabs.find(function (tab) { return tab.active; });
    if (R.isNil(activeTab)) {
        return tabs[0].value;
    }
    return activeTab.value;
};
//# sourceMappingURL=tabpanel.support.js.map