import { AnyT } from '../definitions.interface';
import { ApplicationStorageTypeEnum } from '../storage/storage.interface';
import { IBaseTransportRequestEntity } from '../entities-definitions.interface';
import { IButtonProps } from '../definition';
export interface IApplicationCurrencySettings {
    uiLocale?: string;
    uiCurrency?: string;
}
export interface IApplicationResourcePaths {
    uiDefaultIconPath?: string;
}
/**
 * @stable [02.02.2019]
 */
export interface ITransportSettings extends IBaseTransportRequestEntity {
    binaryContentType?: string;
    formDataContentType?: string;
    binaryUrl?: string;
    apiUrl?: string;
    noCachePrefix?: string;
}
export declare enum StartDayOfWeekT {
    MONDAY = 0,
    SUNDAY = 1
}
export interface IDateTimeSettings {
    startDayOfWeek?: StartDayOfWeekT;
    currentDate?: Date;
    timeZone?: string;
    dateFormat?: string;
    timeFormat?: string;
    dateTimeFormat?: string;
    uiDateFormat?: string;
    uiMonthFormat?: string;
    uiTimeFormat?: string;
    pstDateFormat?: string;
    pstTimeFormat?: string;
    uiDateMask?: Array<string | RegExp>;
    uiTimeMask?: Array<string | RegExp>;
    uiShortTimeMask?: Array<string | RegExp>;
    uiDatePattern?: string;
    uiTimePattern?: string;
    uiShortTimePattern?: string;
}
export interface IApplicationPhoneSettings {
    uiPattern?: string;
    uiMask?: Array<string | RegExp>;
    uiCountryAbbreviation?: string;
}
export interface IApplicationNumberSettings {
    uiPattern?: string;
}
export interface IApplicationMessagesSettings {
    newAppVersionMessageHasBeenDeployed?: string;
    dialogTitleMessage?: string;
    dialogCancelMessage?: string;
    dialogAcceptMessage?: string;
    logoutNotificationMessage?: string;
    accessDeniedMessage?: string;
    sorryMessage?: string;
    waitMessage?: string;
    waitingMessage?: string;
    defaultErrorMessage?: string;
    dataSaved?: string;
    dndMessage?: string;
    errorMessage?: string;
    fileLoadErrorMessage?: string;
    noFileToShowMessage?: string;
    serviceTemporarilyUnavailableMessage?: string;
    noAvailableItemsToSelectMessage?: string;
    emptyMessage?: string;
    emptyDataMessage?: string;
    appNotReadyMessage?: string;
    followingErrorHasOccurredMessage?: string;
    noItemsMessage?: string;
    filterPlaceholderMessage?: string;
    takeSnapshotMessage?: string;
    confirmationMessage?: string;
    continueMessage?: string;
    acceptMessage?: string;
    closeMessage?: string;
    previewMessage?: string;
    addressSelectionMessage?: string;
    putMarkerHereMessage?: string;
    invalidAddressMessage?: string;
    signIntoYourAccountMessage?: string;
    welcomeMessage?: string;
    exportActionTitleMessage?: string;
    refreshActionTitleMessage?: string;
    unknownFileMessage?: string;
    requestCancelErrorMessage?: string;
}
export interface IApplicationAuthorizationSettings {
    isAuthorizationNeeded?: boolean;
}
export interface IApplicationChannelSettings {
    eventToListen?: string;
    eventToEmit?: string;
}
export interface IApplicationGoogleMapsSettings {
    lat?: number;
    lng?: number;
    zoom?: number;
    prettyZoom?: number;
}
export interface IComponentsSettings {
    button?: IButtonProps;
}
export interface ISettings {
    signalRUrl?: string;
    downloadUrl?: string;
    metaFilesJsonUrl?: string;
    pdfWorkerDirectoryUrl?: string;
    emptyPictureUrl?: string;
    companyName?: string;
    companyCountry?: string;
    usePersistence?: boolean;
    transport?: ITransportSettings;
    persistenceStorage?: ApplicationStorageTypeEnum;
    entityEmptyId?: AnyT;
    resourcePaths?: IApplicationResourcePaths;
    dateTime?: IDateTimeSettings;
    phone?: IApplicationPhoneSettings;
    currency?: IApplicationCurrencySettings;
    number?: IApplicationNumberSettings;
    messages?: IApplicationMessagesSettings;
    channel?: IApplicationChannelSettings;
    authorization?: IApplicationAuthorizationSettings;
    googleMaps?: IApplicationGoogleMapsSettings;
    components?: IComponentsSettings;
}
export declare const REGEXP_REPO: {
    price: string;
    number: string;
    digital: string;
};
export declare const DEFAULT_APPLICATION_SETTINGS: ISettings;
